/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.api;

import org.eclipse.emf.diffmerge.patch.api.IModelPatchEntryFilter;
import org.eclipse.emf.diffmerge.patch.api.ModelPatch;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchBuilder;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchEntry;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class ModelPatchFilterApplier {
  /**
   * Filter out matching entries from the originalPatch based on the given filter and return the filtered patch.
   * Matching elements will not be in the new patch. If you need the matching elements use the
   * {@link #applyUnfilter(ModelPatch, IModelPatchEntryFilter) applyUnfilter}.
   * 
   * @param originalModelPatch The modifiable patch
   * @param filter The applicable filter
   * @return ModelPatch which does not contain filtered elements
   */
  public ModelPatch applyFilter(final ModelPatch originalPatch, final IModelPatchEntryFilter filter) {
    final ModelPatchBuilder builder = ModelPatchBuilder.create();
    final Function1<ModelPatchEntry, Boolean> _function = (ModelPatchEntry it) -> {
      boolean _isEntryFiltered = filter.isEntryFiltered(it);
      return Boolean.valueOf((!_isEntryFiltered));
    };
    builder.append(IterableExtensions.<ModelPatchEntry>toList(IterableExtensions.<ModelPatchEntry>filter(originalPatch.getEntries(), _function)));
    return builder.build();
  }
  
  /**
   * Filter out non matching entries from the originalPatch based on the given filter and return the filtered patch.
   * Only matching elements will be in the new patch. If you need the non matching elements
   * {@link #applyFilter(ModelPatch, IModelPatchEntryFilter) applyFilter}.
   * 
   * @param originalModelPatch The modifiable patch
   * @param filter The applicable filter
   * @return ModelPatch which only contains filtered elements
   */
  public ModelPatch applyUnfilter(final ModelPatch originalPatch, final IModelPatchEntryFilter filter) {
    final ModelPatchBuilder builder = ModelPatchBuilder.create();
    final Function1<ModelPatchEntry, Boolean> _function = (ModelPatchEntry it) -> {
      return Boolean.valueOf(filter.isEntryFiltered(it));
    };
    builder.append(IterableExtensions.<ModelPatchEntry>toList(IterableExtensions.<ModelPatchEntry>filter(originalPatch.getEntries(), _function)));
    return builder.build();
  }
}
