/**
 * Copyright (c) 2017-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Abel Hegedus (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.persistence.emf;

import org.apache.log4j.Logger;
import org.eclipse.emf.diffmerge.patch.api.ModelPatch;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchException;
import org.eclipse.emf.diffmerge.patch.persistence.ModelPatchPersister;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EModelPatch;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EModelPatchFactory;
import org.eclipse.emf.diffmerge.patch.persistence.emf.transform.EModelPatchToPOJO;
import org.eclipse.emf.diffmerge.patch.persistence.emf.transform.ModelPatchToEMF;
import org.eclipse.xtext.xbase.lib.Extension;

@SuppressWarnings("all")
public class EMFModelPatchPersister implements ModelPatchPersister<EModelPatch> {
  @Extension
  private EModelPatchFactory factory = EModelPatchFactory.eINSTANCE;
  
  private final Logger logger = Logger.getLogger(EMFModelPatchPersister.class);
  
  @Override
  public EModelPatch persist(final ModelPatch modelPatch) throws ModelPatchException {
    final EModelPatch root = this.factory.createEModelPatch();
    final ModelPatchToEMF mp2emf = new ModelPatchToEMF(modelPatch, root, this.logger, true);
    mp2emf.transform();
    return root;
  }
  
  @Override
  public ModelPatch load(final EModelPatch eModelPatch) throws ModelPatchException {
    final ModelPatch root = new ModelPatch();
    final EModelPatchToPOJO mp2pojo = new EModelPatchToPOJO(eModelPatch, root, this.logger);
    mp2pojo.transform();
    return root;
  }
}
