/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Balazs Grill, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.runtime.modelaccess;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.diffmerge.patch.runtime.modelaccess.EMFModelAccess;
import org.eclipse.emf.diffmerge.patch.runtime.modelaccess.ModelAccessTypes;
import org.eclipse.emf.diffmerge.patch.runtime.modelaccess.SimpleReflectiveEMFModelAccess;
import org.eclipse.emf.diffmerge.patch.runtime.modelaccess.ViatraModelAccess;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.emf.EMFScope;
import org.eclipse.viatra.transformation.runtime.emf.modelmanipulation.ModelManipulationWithEditingDomain;
import org.eclipse.viatra.transformation.runtime.emf.modelmanipulation.SimpleModelManipulations;
import org.eclipse.xtext.xbase.lib.Exceptions;

@SuppressWarnings("all")
public class ModelAccessProvider {
  public EMFModelAccess getSelectedModelAccess(final String pref, final ResourceSet resourceSet) {
    final ModelAccessTypes accessType = ModelAccessTypes.valueOf(pref);
    return this.getSelectedModelAccess(accessType, resourceSet);
  }
  
  public EMFModelAccess getSelectedModelAccess(final ModelAccessTypes accessType, final ResourceSet resourceSet) {
    try {
      EMFModelAccess _xblockexpression = null;
      {
        EMFModelAccess access = null;
        EMFModelAccess _switchResult = null;
        if (accessType != null) {
          switch (accessType) {
            case EMF_REFLECTIVE:
              SimpleReflectiveEMFModelAccess _simpleReflectiveEMFModelAccess = new SimpleReflectiveEMFModelAccess();
              _switchResult = access = _simpleReflectiveEMFModelAccess;
              break;
            case VIATRA:
              EMFModelAccess _xblockexpression_1 = null;
              {
                final Resource temporaryResource = resourceSet.createResource(URI.createURI("temporary_resource"));
                final EMFScope scope = new EMFScope(resourceSet);
                final ViatraQueryEngine engine = ViatraQueryEngine.on(scope);
                final EditingDomain domain = AdapterFactoryEditingDomain.getEditingDomainFor(resourceSet);
                EMFModelAccess _xifexpression = null;
                if ((domain != null)) {
                  EMFModelAccess _xblockexpression_2 = null;
                  {
                    final ModelManipulationWithEditingDomain viatraModelManipulator = new ModelManipulationWithEditingDomain(engine, domain);
                    ViatraModelAccess _viatraModelAccess = new ViatraModelAccess(viatraModelManipulator, temporaryResource);
                    _xblockexpression_2 = access = _viatraModelAccess;
                  }
                  _xifexpression = _xblockexpression_2;
                } else {
                  EMFModelAccess _xblockexpression_3 = null;
                  {
                    final SimpleModelManipulations viatraModelManipulator = new SimpleModelManipulations(engine);
                    ViatraModelAccess _viatraModelAccess = new ViatraModelAccess(viatraModelManipulator, temporaryResource);
                    _xblockexpression_3 = access = _viatraModelAccess;
                  }
                  _xifexpression = _xblockexpression_3;
                }
                _xblockexpression_1 = _xifexpression;
              }
              _switchResult = _xblockexpression_1;
              break;
            default:
              SimpleReflectiveEMFModelAccess _simpleReflectiveEMFModelAccess_1 = new SimpleReflectiveEMFModelAccess();
              _switchResult = access = _simpleReflectiveEMFModelAccess_1;
              break;
          }
        } else {
          SimpleReflectiveEMFModelAccess _simpleReflectiveEMFModelAccess_1 = new SimpleReflectiveEMFModelAccess();
          _switchResult = access = _simpleReflectiveEMFModelAccess_1;
        }
        _xblockexpression = _switchResult;
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
