/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Balazs Grill, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.ui.utils;

import com.google.common.base.Objects;
import java.util.ArrayList;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.wizards.newresource.BasicNewFileResourceWizard;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class DialogFactory {
  private final Shell shell;
  
  public DialogFactory(final Shell shell) {
    this.shell = shell;
  }
  
  public void openInformationDialog(final String title, final String message) {
    MessageDialog.openInformation(this.shell, title, message);
  }
  
  public String openFileDialog(final String[] supportedFileExtensions) {
    return this.openFileDialog("File Selection", supportedFileExtensions);
  }
  
  public String openFileDialog(final String title, final String[] supportedFileExtensions) {
    FileDialog _fileDialog = new FileDialog(this.shell, SWT.OPEN);
    final Procedure1<FileDialog> _function = (FileDialog it) -> {
      it.setText(title);
      it.setFilterExtensions(supportedFileExtensions);
    };
    return ObjectExtensions.<FileDialog>operator_doubleArrow(_fileDialog, _function).open();
  }
  
  public IFile openSaveFileDialog(final IWorkbench workbench, final IStructuredSelection selection, final String fextension) {
    abstract class __DialogFactory_1 extends BasicNewFileResourceWizard {
      String path;
      
      IFile file;
      
      WizardNewFileCreationPage mainPage;
    }
    
    __DialogFactory_1 ___DialogFactory_1 = new __DialogFactory_1() {
      {
        path = null;
        
        file = null;
      }
      @Override
      public void addPages() {
        super.addPages();
        IWizardPage _get = this.getPages()[0];
        this.mainPage = ((WizardNewFileCreationPage) _get);
        this.mainPage.setFileExtension(fextension);
      }
      
      @Override
      public boolean performFinish() {
        this.file = this.mainPage.createNewFile();
        boolean _equals = Objects.equal(this.file, null);
        if (_equals) {
          return false;
        }
        this.selectAndReveal(this.file);
        IPath _location = this.file.getLocation();
        boolean _notEquals = (!Objects.equal(_location, null));
        if (_notEquals) {
          this.path = this.file.getLocation().toString();
        }
        return true;
      }
    };
    final Procedure1<__DialogFactory_1> _function = (__DialogFactory_1 it) -> {
      it.setWindowTitle("Modelpatch file");
    };
    final __DialogFactory_1 wizard = ObjectExtensions.<__DialogFactory_1>operator_doubleArrow(___DialogFactory_1, _function);
    wizard.init(workbench, selection);
    WizardDialog _wizardDialog = new WizardDialog(this.shell, wizard);
    final Procedure1<WizardDialog> _function_1 = (WizardDialog it) -> {
      it.setBlockOnOpen(true);
    };
    final WizardDialog dialog = ObjectExtensions.<WizardDialog>operator_doubleArrow(_wizardDialog, _function_1);
    dialog.open();
    return wizard.file;
  }
  
  public IEditorPart openModelPatchEditor(final IWorkbenchWindow window, final IFile file) {
    IEditorPart _xtrycatchfinallyexpression = null;
    try {
      IEditorPart _xifexpression = null;
      boolean _notEquals = (!Objects.equal(window, null));
      if (_notEquals) {
        IEditorPart _xblockexpression = null;
        {
          final IWorkbenchPage page = window.getActivePage();
          IEditorPart _xifexpression_1 = null;
          boolean _notEquals_1 = (!Objects.equal(page, null));
          if (_notEquals_1) {
            _xifexpression_1 = IDE.openEditor(page, file, true);
          }
          _xblockexpression = _xifexpression_1;
        }
        _xifexpression = _xblockexpression;
      }
      _xtrycatchfinallyexpression = _xifexpression;
    } catch (final Throwable _t) {
      if (_t instanceof PartInitException) {
        final PartInitException e = (PartInitException)_t;
        this.openError(window.getShell(), "Modelpatch editor cannot be opened", e.getMessage(), e);
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    return _xtrycatchfinallyexpression;
  }
  
  public String openWorkspaceFileDialog() {
    return this.openWorkspaceFileDialog("File Selection");
  }
  
  public String openWorkspaceFileDialog(final String title) {
    WorkbenchLabelProvider _workbenchLabelProvider = new WorkbenchLabelProvider();
    WorkbenchContentProvider _workbenchContentProvider = new WorkbenchContentProvider();
    final ElementTreeSelectionDialog dialog = new ElementTreeSelectionDialog(this.shell, _workbenchLabelProvider, _workbenchContentProvider);
    dialog.setTitle(title);
    dialog.setInput(ResourcesPlugin.getWorkspace().getRoot());
    dialog.setAllowMultiple(false);
    int _open = dialog.open();
    boolean _equals = (_open == Window.OK);
    if (_equals) {
      Object _firstResult = dialog.getFirstResult();
      return ((IResource) _firstResult).getLocation().toOSString();
    }
    return null;
  }
  
  public int openErrorDialog(final String dialogTitle, final String message, final String errorMessage, final Throwable ex, final String ID) {
    return ErrorDialog.openError(this.shell, dialogTitle, message, this.createMultiStatus(errorMessage, ex, ID));
  }
  
  public int openErrorDialog(final String dialogTitle, final String message, final Throwable ex, final String ID) {
    return ErrorDialog.openError(this.shell, dialogTitle, message, this.createMultiStatus(ex.getMessage(), ex, ID));
  }
  
  public void openErrorDialog(final String dialogTitle, final String message) {
    MessageDialog.openError(this.shell, dialogTitle, message);
  }
  
  /**
   * Copied from {@link org.eclipse.ui.internal.ide.DialogUtil#openError(Shell, String, String, PartInitException)}
   */
  private void openError(final Shell parent, final String title, final String message, final PartInitException exception) {
    CoreException nestedException = null;
    IStatus status = exception.getStatus();
    if (((status != null) && (status.getException() instanceof CoreException))) {
      Throwable _exception = status.getException();
      nestedException = ((CoreException) _exception);
    }
    if ((nestedException != null)) {
      ErrorDialog.openError(parent, title, message, nestedException.getStatus());
    } else {
      MessageDialog.openError(parent, title, message);
    }
  }
  
  private MultiStatus createMultiStatus(final String msg, final Throwable t, final String ID) {
    final ArrayList<Status> childStatuses = CollectionLiterals.<Status>newArrayList();
    final StackTraceElement[] stackTraces = t.getStackTrace();
    for (final StackTraceElement stackTrace : stackTraces) {
      {
        String _string = stackTrace.toString();
        final Status status = new Status(IStatus.ERROR, ID, _string);
        childStatuses.add(status);
      }
    }
    IStatus[] _array = childStatuses.<IStatus>toArray(new IStatus[] {});
    String _string = t.toString();
    return new MultiStatus(ID, IStatus.ERROR, _array, _string, t);
  }
}
