/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.ui.utils;

import com.google.common.base.Objects;
import java.util.List;
import org.eclipse.emf.diffmerge.patch.persistence.emf.EMFModelPatchSerializer;
import org.eclipse.emf.diffmerge.patch.serializer.IModelPatchSerializer;
import org.eclipse.emf.diffmerge.patch.serializer.gson.GsonModelPatchSerializer;
import org.eclipse.emf.diffmerge.patch.ui.utils.PersistenceTypes;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;

@SuppressWarnings("all")
public class SerializerProvider {
  public IModelPatchSerializer getSelectedSerializer(final String typeName) {
    final PersistenceTypes accessType = PersistenceTypes.valueOf(typeName);
    return this.getSelectedSerializer(accessType);
  }
  
  public IModelPatchSerializer getSelectedSerializer(final PersistenceTypes type) {
    IModelPatchSerializer serializer = null;
    if (type != null) {
      switch (type) {
        case GSON:
          serializer = GsonModelPatchSerializer.create(this.getClass().getClassLoader());
          break;
        case EMF:
          EMFModelPatchSerializer _eMFModelPatchSerializer = new EMFModelPatchSerializer();
          serializer = _eMFModelPatchSerializer;
          break;
        default:
          serializer = GsonModelPatchSerializer.create(this.getClass().getClassLoader());
          break;
      }
    } else {
      serializer = GsonModelPatchSerializer.create(this.getClass().getClassLoader());
    }
    return serializer;
  }
  
  public IModelPatchSerializer getSerializerByFileExtension(final String fileExtension) {
    final Function1<PersistenceTypes, IModelPatchSerializer> _function = (PersistenceTypes it) -> {
      return this.getSelectedSerializer(it);
    };
    final Function1<IModelPatchSerializer, Boolean> _function_1 = (IModelPatchSerializer it) -> {
      String _preferredFileExtension = it.getPreferredFileExtension();
      return Boolean.valueOf(Objects.equal(_preferredFileExtension, fileExtension));
    };
    final IModelPatchSerializer serializer = IterableExtensions.<IModelPatchSerializer>findFirst(ListExtensions.<PersistenceTypes, IModelPatchSerializer>map(((List<PersistenceTypes>)Conversions.doWrapArray(PersistenceTypes.values())), _function), _function_1);
    return serializer;
  }
}
